@@ -21,6 +21,9 @@
 #include <fcntl.h>
 #endif
 
+#define MQTT_DEFAULT_MAX_BUF_SIZE  (25 * 1024 * 1024)
+#define MQTT_PARENT_MAX_BUF_SIZE  (128 * 1024 * 1024)
+
 int aclk_pubacks_per_conn = 0; // How many PubAcks we got since MQTT conn est.
 int aclk_rcvd_cloud_msgs = 0;
 int aclk_connection_counter = 0;
@@ -645,9 +648,6 @@ static int aclk_attempt_to_connect(mqtt_wss_client client)
             continue;
         }
 
-        struct mqtt_wss_proxy proxy_conf = { .host = NULL, .port = 0, .username = NULL, .password = NULL, .type = MQTT_WSS_DIRECT };
-        aclk_set_proxy((char**)&proxy_conf.host, &proxy_conf.port, (char**)&proxy_conf.username, (char**)&proxy_conf.password, &proxy_conf.type);
-
         struct mqtt_connect_params mqtt_conn_params = {
             .clientid   = "anon",
             .username   = "anon",
@@ -670,6 +670,9 @@ static int aclk_attempt_to_connect(mqtt_wss_client client)
         url_t_destroy(&base_url);
         if (rc != HTTPS_CLIENT_RESP_OK) {
             aclk_status_set((ACLK_STATUS)rc);
+            aclk_env_t_destroy(aclk_env);
+            freez(aclk_env);
+            aclk_env = NULL;
             continue;
         }
 
@@ -728,9 +731,9 @@ static int aclk_attempt_to_connect(mqtt_wss_client client)
         }
 
         memset(&mqtt_url, 0, sizeof(url_t));
-        if (url_parse(aclk_env->transports[rc]->endpoint, &mqtt_url)){
+        if (url_parse(aclk_env->transports[trp]->endpoint, &mqtt_url)){
             aclk_status_set(ACLK_STATUS_INVALID_ENV_TRANSPORT_URL);
-            error_report("ACLK: failed to parse target URL for /env trp idx %d \"%s\"", trp, aclk_env->transports[rc]->endpoint);
+            error_report("ACLK: failed to parse target URL for /env trp idx %d \"%s\"", trp, aclk_env->transports[trp]->endpoint);
             url_t_destroy(&mqtt_url);
             continue;
         }
@@ -744,6 +747,9 @@ static int aclk_attempt_to_connect(mqtt_wss_client client)
 
         int ssl_flags = cloud_config_insecure_get() ? MQTT_WSS_SSL_ALLOW_SELF_SIGNED : MQTT_WSS_SSL_CERT_CHECK_FULL;
 
+        struct mqtt_wss_proxy proxy_conf = { .host = NULL, .port = 0, .username = NULL, .password = NULL, .type = MQTT_WSS_DIRECT };
+        aclk_set_proxy((char**)&proxy_conf.host, &proxy_conf.port, (char**)&proxy_conf.username, (char**)&proxy_conf.password, &proxy_conf.type);
+
 #ifdef ACLK_DISABLE_CHALLENGE
         int mqtt_rc = mqtt_wss_connect(client, base_url.host, base_url.port, &mqtt_conn_params, ssl_flags, &proxy_conf);
         url_t_destroy(&base_url);
@@ -861,9 +867,8 @@ void *aclk_main(void *ptr)
 #endif
 
     // Enable MQTT buffer growth if necessary
-    // e.g. old cloud architecture clients with huge nodes
-    // that send JSON payloads of 10 MB as single messages
-    mqtt_wss_set_max_buf_size(mqttwss_client, 25*1024*1024);
+    size_t max_buf_size = netdata_conf_is_parent() ? MQTT_PARENT_MAX_BUF_SIZE : MQTT_DEFAULT_MAX_BUF_SIZE;
+    mqtt_wss_set_max_buf_size(mqttwss_client, max_buf_size);
 
     // Keep reconnecting and talking until our time has come
     // and the Grim Reaper (exit_initiated) calls
@@ -947,7 +952,7 @@ void aclk_create_node_instance_job(RRDHOST *host)
     if (!claim_id_is_set(claim_id))
         return;
 
-    aclk_query_t query = aclk_query_new(REGISTER_NODE);
+    aclk_query_t *query = aclk_query_new(REGISTER_NODE);
     int32_t hops =  rrdhost_ingestion_hops(host);
     node_instance_creation_t node_instance_creation = {
         .hops = hops,
@@ -974,7 +979,7 @@ void aclk_update_node_instance_job(RRDHOST *host, int live, int queryable)
     if (!claim_id_is_set(claim_id))
         return;
 
-    aclk_query_t query = aclk_query_new(NODE_STATE_UPDATE);
+    aclk_query_t *query = aclk_query_new(NODE_STATE_UPDATE);
 
     int32_t hops = rrdhost_ingestion_hops(host);
     node_instance_connection_t node_state_update = {
