@@ -125,8 +125,6 @@ static inline int aclk_v2_payload_get_query(const char *payload, char **query_ur
 
 static int aclk_handle_cloud_http_request_v2(struct aclk_request *cloud_to_agent, char *raw_payload)
 {
-    aclk_query_t query;
-
     errno_clear();
     if (cloud_to_agent->version < ACLK_V_COMPRESSION) {
         netdata_log_error(
@@ -136,7 +134,7 @@ static int aclk_handle_cloud_http_request_v2(struct aclk_request *cloud_to_agent
         return 1;
     }
 
-    query = aclk_query_new(HTTP_API_V2);
+    aclk_query_t *query = aclk_query_new(HTTP_API_V2);
 
     if (unlikely(aclk_extract_v2_data(raw_payload, &query->data.http_api_v2.payload))) {
         netdata_log_error("Error extracting payload expected after the JSON dictionary.");
@@ -255,7 +253,8 @@ int create_node_instance_result(const char *msg, size_t msg_len)
 
     netdata_log_debug(D_ACLK, "CreateNodeInstanceResult: guid:%s nodeid:%s", res.machine_guid, res.node_id);
 
-    aclk_query_t query = aclk_query_new(CREATE_NODE_INSTANCE);
+    aclk_query_t *query = aclk_query_new(CREATE_NODE_INSTANCE);
+
     query->data.node_id = res.node_id;          // Will be freed on query free
     query->machine_guid = res.machine_guid;     // Will be freed on query free
     aclk_add_job(query);
@@ -266,7 +265,7 @@ int send_node_instances(const char *msg, size_t msg_len)
 {
     UNUSED(msg);
     UNUSED(msg_len);
-    aclk_query_t query = aclk_query_new(SEND_NODE_INSTANCES);
+    aclk_query_t *query = aclk_query_new(SEND_NODE_INSTANCES);
     aclk_add_job(query);
     return 0;
 }
@@ -302,7 +301,7 @@ int start_alarm_streaming(const char *msg, size_t msg_len)
         netdata_log_error("Error parsing StartAlarmStreaming");
         return 1;
     }
-    aclk_query_t query = aclk_query_new(ALERT_START_STREAMING);
+    aclk_query_t *query = aclk_query_new(ALERT_START_STREAMING);
     query->data.node_id = res.node_id;      // Will be freed on query free
     query->version = res.version;
     aclk_add_job(query);
@@ -318,7 +317,7 @@ int send_alarm_checkpoint(const char *msg, size_t msg_len)
         freez(sac.claim_id);
         return 1;
     }
-    aclk_query_t query = aclk_query_new(ALERT_CHECKPOINT);
+    aclk_query_t *query = aclk_query_new(ALERT_CHECKPOINT);
     query->data.node_id = sac.node_id;  // Will be freed on query free
     query->claim_id = sac.claim_id;
     query->version = sac.version;
@@ -347,7 +346,7 @@ int send_alarm_snapshot(const char *msg, size_t msg_len)
         destroy_send_alarm_snapshot(sas);
         return 1;
     }
-    aclk_query_t query = aclk_query_new(ALERT_CHECKPOINT);
+    aclk_query_t *query = aclk_query_new(ALERT_CHECKPOINT);
     query->data.node_id = sas->node_id;     // Will be freed on query free
     query->claim_id = sas->claim_id;        // Will be freed on query free
     query->version = 0; // force snapshot
@@ -386,7 +385,7 @@ int contexts_checkpoint(const char *msg, size_t msg_len)
     if (!cmd)
         return 1;
 
-    aclk_query_t query = aclk_query_new(CTX_CHECKPOINT);
+    aclk_query_t *query = aclk_query_new(CTX_CHECKPOINT);
     query->data.payload = cmd;
     aclk_add_job(query);
     return 0;
@@ -403,7 +402,7 @@ int stop_streaming_contexts(const char *msg, size_t msg_len)
     if (!cmd)
         return 1;
 
-    aclk_query_t query = aclk_query_new(CTX_STOP_STREAMING);
+    aclk_query_t *query = aclk_query_new(CTX_STOP_STREAMING);
     query->data.payload = cmd;
     aclk_add_job(query);
     return 0;
