@@ -261,7 +261,7 @@ struct cgroup {
 };
 
 struct discovery_thread {
-    uv_thread_t thread;
+    ND_THREAD *thread;
     uv_mutex_t mutex;
     uv_cond_t cond_var;
     int exited;
@@ -274,7 +274,7 @@ extern char cgroup_chart_id_prefix[];
 extern char services_chart_id_prefix[];
 extern uv_mutex_t cgroup_root_mutex;
 
-void cgroup_discovery_worker(void *ptr);
+void *cgroup_discovery_worker(void *ptr);
 
 extern bool is_inside_k8s;
 extern long system_page_size;
@@ -378,7 +378,7 @@ static inline char *cgroup_chart_type(char *buffer, struct cgroup *cg) {
     buffer[0] = '\0';
 
     if (cg->chart_id[0] == '\0' || (cg->chart_id[0] == '/' && cg->chart_id[1] == '\0'))
-        strncpy(buffer, "cgroup_root", RRD_ID_LENGTH_MAX);
+        strncpyz(buffer, "cgroup_root", RRD_ID_LENGTH_MAX - 1);
     else if (is_cgroup_systemd_service(cg))
         snprintfz(buffer, RRD_ID_LENGTH_MAX, "%s%s", services_chart_id_prefix, cg->chart_id);
     else
@@ -387,8 +387,8 @@ static inline char *cgroup_chart_type(char *buffer, struct cgroup *cg) {
     return buffer;
 }
 
-#define RRDFUNCTIONS_CGTOP_HELP "View running containers"
-#define RRDFUNCTIONS_SYSTEMD_SERVICES_HELP "View systemd services"
+#define RRDFUNCTIONS_CGTOP_HELP "Lists active containers and cgroups with resource usage including CPU, memory, disk I/O, and network traffic."
+#define RRDFUNCTIONS_SYSTEMD_SERVICES_HELP "Shows systemd service cgroups with their process counts and resource consumption (CPU, memory, I/O)."
 
 int cgroup_function_cgroup_top(BUFFER *wb, const char *function, BUFFER *payload, const char *source);
 int cgroup_function_systemd_top(BUFFER *wb, const char *function, BUFFER *payload, const char *source);
