@@ -357,9 +357,12 @@ extern RRDHOST *localhost;
 #define rrdhost_sender_replicating_charts_minus_one(host) (__atomic_sub_fetch(&((host)->stream.snd.status.replication.charts), 1, __ATOMIC_RELAXED))
 #define rrdhost_sender_replicating_charts_zero(host) (__atomic_store_n(&((host)->stream.snd.status.replication.charts), 0, __ATOMIC_RELAXED))
 
+#define rrdhost_is_virtual(host)                                                                                \
+    rrdhost_option_check(host, RRDHOST_OPTION_VIRTUAL_HOST)
+
 #define rrdhost_is_local(host)  ( \
     (host) == localhost ||                                                                                      \
-    rrdhost_option_check(host, RRDHOST_OPTION_VIRTUAL_HOST)                                                     \
+    rrdhost_is_virtual(host)                                                                                    \
     )
 
 #define rrdhost_is_online_flags(flags) ((flags & RRDHOST_FLAG_COLLECTOR_ONLINE) && !(flags & RRDHOST_FLAG_ORPHAN))
@@ -389,7 +392,7 @@ void rrdhost_acquired_release(RRDHOST_ACQUIRED *rha);
 
 RRDHOST *rrdhost_find_by_hostname(const char *hostname);
 RRDHOST *rrdhost_find_by_guid(const char *guid);
-RRDHOST *rrdhost_find_by_node_id(char *node_id);
+RRDHOST *rrdhost_find_by_node_id(const char *node_id);
 
 #ifdef RRDHOST_INTERNALS
 RRDHOST *rrdhost_create(
