@@ -106,6 +106,7 @@ static void exporting_clean_engine()
         instance = instance->next;
 
         clean_instance(current_instance);
+        freez(current_instance);
     }
 
     freez((void *)engine->config.hostname);
@@ -195,9 +196,6 @@ void *exporting_main(void *ptr)
 
     if (init_connectors(engine) != 0) {
         netdata_log_error("EXPORTING: cannot initialize exporting connectors");
-
-        analytics_statistic_t statistic = { "EXPORTING_START", "FAIL", "-" };
-        analytics_statistic_send(&statistic);
         goto cleanup;
     }
 
@@ -222,6 +220,7 @@ void *exporting_main(void *ptr)
         return NULL;
 #endif
     }
+    service_exits();
 
 cleanup:
     return NULL;
