@@ -8,12 +8,14 @@ import (
 	"errors"
 	"fmt"
 
+	"github.com/gosnmp/gosnmp"
+
 	"github.com/netdata/netdata/go/plugins/pkg/matcher"
 	"github.com/netdata/netdata/go/plugins/plugin/go.d/agent/discovery/sd/discoverer/snmpsd"
 	"github.com/netdata/netdata/go/plugins/plugin/go.d/agent/module"
 	"github.com/netdata/netdata/go/plugins/plugin/go.d/agent/vnodes"
-
-	"github.com/gosnmp/gosnmp"
+	"github.com/netdata/netdata/go/plugins/plugin/go.d/collector/snmp/ddsnmp"
+	"github.com/netdata/netdata/go/plugins/plugin/go.d/collector/snmp/ddsnmp/ddsnmpcollector"
 )
 
 //go:embed "config_schema.json"
@@ -33,8 +35,9 @@ func init() {
 func New() *Collector {
 	return &Collector{
 		Config: Config{
-			CreateVnode: true,
-			Community:   "public",
+			CreateVnode:                true,
+			EnableProfiles:             true,
+			EnableProfilesTableMetrics: true,
 			Options: Options{
 				Port:           161,
 				Retries:        1,
@@ -59,6 +62,9 @@ func New() *Collector {
 		checkMaxReps:  true,
 		collectIfMib:  true,
 		netInterfaces: make(map[string]*netInterface),
+
+		seenScalarMetrics: make(map[string]bool),
+		seenTableMetrics:  make(map[string]bool),
 	}
 }
 
@@ -66,8 +72,6 @@ type Collector struct {
 	module.Base
 	Config `yaml:",inline" json:""`
 
-	enableProfiles bool
-
 	vnode *vnodes.VirtualNode
 
 	charts      *module.Charts
@@ -75,6 +79,7 @@ type Collector struct {
 
 	newSnmpClient func() gosnmp.Handler
 	snmpClient    gosnmp.Handler
+	ddSnmpColl    *ddsnmpcollector.Collector
 
 	netIfaceFilterByName matcher.Matcher
 	netIfaceFilterByType matcher.Matcher
@@ -87,6 +92,11 @@ type Collector struct {
 	sysInfo *snmpsd.SysInfo
 
 	customOids []string
+
+	snmpProfiles []*ddsnmp.Profile
+
+	seenScalarMetrics map[string]bool
+	seenTableMetrics  map[string]bool
 }
 
 func (c *Collector) Configuration() any {
