@@ -18,7 +18,15 @@
 #define WORKER_TRAIN_FLUSH_MODELS      7
 
 sqlite3 *ml_db = NULL;
-static netdata_mutex_t db_mutex = NETDATA_MUTEX_INITIALIZER;
+static netdata_mutex_t db_mutex;
+
+static void __attribute__((constructor)) init_mutex(void) {
+    netdata_mutex_init(&db_mutex);
+}
+
+static void __attribute__((destructor)) destroy_mutex(void) {
+    netdata_mutex_destroy(&db_mutex);
+}
 
 typedef struct {
     // First/last entry of the dimension in DB when generating the response
@@ -992,7 +1000,7 @@ static void ml_flush_pending_models(ml_worker_t *worker) {
     int op_no = 1;
 
     // begin transaction
-    int rc = db_execute(ml_db, "BEGIN TRANSACTION;");
+    int rc = db_execute(ml_db, "BEGIN TRANSACTION;", NULL);
 
     // add/delete models
     if (!rc) {
@@ -1019,14 +1027,14 @@ static void ml_flush_pending_models(ml_worker_t *worker) {
     // commit transaction
     if (!rc) {
         op_no++;
-        rc = db_execute(ml_db, "COMMIT TRANSACTION;");
+        rc = db_execute(ml_db, "COMMIT TRANSACTION;", NULL);
     }
 
     // rollback transaction on failure
     if (rc) {
         netdata_log_error("Trying to rollback ML transaction because it failed with rc=%d, op_no=%d", rc, op_no);
         op_no++;
-        rc = db_execute(ml_db, "ROLLBACK;");
+        rc = db_execute(ml_db, "ROLLBACK;", NULL);
         if (rc)
             netdata_log_error("ML transaction rollback failed with rc=%d", rc);
     }
